@@ -30,9 +30,10 @@
 //! extern crate lru;
 //!
 //! use lru::LruCache;
+//! use std::num::NonZeroUsize;
 //!
 //! fn main() {
-//!         let mut cache = LruCache::new(2);
+//!         let mut cache = LruCache::new(NonZeroUsize::new(2).unwrap());
 //!         cache.put("apple", 3);
 //!         cache.put("banana", 2);
 //!
@@ -72,6 +73,7 @@ use core::hash::{BuildHasher, Hash, Hasher};
 use core::iter::FusedIterator;
 use core::marker::PhantomData;
 use core::mem;
+use core::num::NonZeroUsize;
 use core::ptr;
 use core::usize;
 
@@ -188,9 +190,9 @@ pub type DefaultHasher = std::collections::hash_map::RandomState;
 /// An LRU Cache
 pub struct LruCache<K, V, S = DefaultHasher> {
     map: HashMap<KeyRef<K>, Box<LruEntry<K, V>>, S>,
-    cap: usize,
+    cap: NonZeroUsize,
 
-    // head and tail are sigil nodes to faciliate inserting entries
+    // head and tail are sigil nodes to facilitate inserting entries
     head: *mut LruEntry<K, V>,
     tail: *mut LruEntry<K, V>,
 }
@@ -202,10 +204,11 @@ impl<K: Hash + Eq, V> LruCache<K, V> {
     ///
     /// ```
     /// use lru::LruCache;
-    /// let mut cache: LruCache<isize, &str> = LruCache::new(10);
+    /// use std::num::NonZeroUsize;
+    /// let mut cache: LruCache<isize, &str> = LruCache::new(NonZeroUsize::new(10).unwrap());
     /// ```
-    pub fn new(cap: usize) -> LruCache<K, V> {
-        LruCache::construct(cap, HashMap::with_capacity(cap))
+    pub fn new(cap: NonZeroUsize) -> LruCache<K, V> {
+        LruCache::construct(cap, HashMap::with_capacity(cap.get()))
     }
 
     /// Creates a new LRU Cache that never automatically evicts items.
@@ -214,10 +217,11 @@ impl<K: Hash + Eq, V> LruCache<K, V> {
     ///
     /// ```
     /// use lru::LruCache;
+    /// use std::num::NonZeroUsize;
     /// let mut cache: LruCache<isize, &str> = LruCache::unbounded();
     /// ```
     pub fn unbounded() -> LruCache<K, V> {
-        LruCache::construct(usize::MAX, HashMap::default())
+        LruCache::construct(NonZeroUsize::new(usize::MAX).unwrap(), HashMap::default())
     }
 }
 
@@ -229,12 +233,16 @@ impl<K: Hash + Eq, V, S: BuildHasher> LruCache<K, V, S> {
     ///
     /// ```
     /// use lru::{LruCache, DefaultHasher};
+    /// use std::num::NonZeroUsize;
     ///
     /// let s = DefaultHasher::default();
-    /// let mut cache: LruCache<isize, &str> = LruCache::with_hasher(10, s);
+    /// let mut cache: LruCache<isize, &str> = LruCache::with_hasher(NonZeroUsize::new(10).unwrap(), s);
     /// ```
-    pub fn with_hasher(cap: usize, hash_builder: S) -> LruCache<K, V, S> {
-        LruCache::construct(cap, HashMap::with_capacity_and_hasher(cap, hash_builder))
+    pub fn with_hasher(cap: NonZeroUsize, hash_builder: S) -> LruCache<K, V, S> {
+        LruCache::construct(
+            cap,
+            HashMap::with_capacity_and_hasher(cap.into(), hash_builder),
+        )
     }
 
     /// Creates a new LRU Cache that never automatically evicts items and
@@ -249,11 +257,17 @@ impl<K: Hash + Eq, V, S: BuildHasher> LruCache<K, V, S> {
     /// let mut cache: LruCache<isize, &str> = LruCache::unbounded_with_hasher(s);
     /// ```
     pub fn unbounded_with_hasher(hash_builder: S) -> LruCache<K, V, S> {
-        LruCache::construct(usize::MAX, HashMap::with_hasher(hash_builder))
+        LruCache::construct(
+            NonZeroUsize::new(usize::MAX).unwrap(),
+            HashMap::with_hasher(hash_builder),
+        )
     }
 
     /// Creates a new LRU Cache with the given capacity.
-    fn construct(cap: usize, map: HashMap<KeyRef<K>, Box<LruEntry<K, V>>, S>) -> LruCache<K, V, S> {
+    fn construct(
+        cap: NonZeroUsize,
+        map: HashMap<KeyRef<K>, Box<LruEntry<K, V>>, S>,
+    ) -> LruCache<K, V, S> {
         // NB: The compiler warns that cache does not need to be marked as mutable if we
         // declare it as such since we only mutate it inside the unsafe block.
         let cache = LruCache {
@@ -278,7 +292,8 @@ impl<K: Hash + Eq, V, S: BuildHasher> LruCache<K, V, S> {
     ///
     /// ```
     /// use lru::LruCache;
-    /// let mut cache = LruCache::new(2);
+    /// use std::num::NonZeroUsize;
+    /// let mut cache = LruCache::new(NonZeroUsize::new(2).unwrap());
     ///
     /// assert_eq!(None, cache.put(1, "a"));
     /// assert_eq!(None, cache.put(2, "b"));
@@ -299,7 +314,8 @@ impl<K: Hash + Eq, V, S: BuildHasher> LruCache<K, V, S> {
     ///
     /// ```
     /// use lru::LruCache;
-    /// let mut cache = LruCache::new(2);
+    /// use std::num::NonZeroUsize;
+    /// let mut cache = LruCache::new(NonZeroUsize::new(2).unwrap());
     ///
     /// assert_eq!(None, cache.push(1, "a"));
     /// assert_eq!(None, cache.push(2, "b"));
@@ -318,9 +334,9 @@ impl<K: Hash + Eq, V, S: BuildHasher> LruCache<K, V, S> {
         self.capturing_put(k, v, true)
     }
 
-    // Used internally by `put` and `push` to add a new entry to the lru. 
+    // Used internally by `put` and `push` to add a new entry to the lru.
     // Takes ownership of and returns entries replaced due to the cache's capacity
-    // when `capture` is true. 
+    // when `capture` is true.
     fn capturing_put(&mut self, k: K, mut v: V, capture: bool) -> Option<(K, V)> {
         let node_ref = self.map.get_mut(&KeyRef { k: &k });
 
@@ -336,11 +352,6 @@ impl<K: Hash + Eq, V, S: BuildHasher> LruCache<K, V, S> {
                 Some((k, v))
             }
             None => {
-                // if the capacity is zero, do nothing
-                if self.cap() == 0 {
-                    return None;
-                }
-
                 let (replaced, mut node) = self.replace_or_create_node(k, v);
 
                 let node_ptr: *mut LruEntry<K, V> = &mut *node;
@@ -354,12 +365,11 @@ impl<K: Hash + Eq, V, S: BuildHasher> LruCache<K, V, S> {
         }
     }
 
-    
-
     // Used internally to swap out a node if the cache is full or to create a new node if space
-    // is available. Shared between `put`, `push`, and `get_or_insert`.
+    // is available. Shared between `put`, `push`, `get_or_insert`, and `get_or_insert_mut`.
+    #[allow(clippy::type_complexity)]
     fn replace_or_create_node(&mut self, k: K, v: V) -> (Option<(K, V)>, Box<LruEntry<K, V>>) {
-        if self.len() == self.cap() {
+        if self.len() == self.cap().get() {
             // if the cache is full, remove the last entry so we can use it for the new key
             let old_key = KeyRef {
                 k: unsafe { &(*(*(*self.tail).prev).key.as_ptr()) },
@@ -389,7 +399,8 @@ impl<K: Hash + Eq, V, S: BuildHasher> LruCache<K, V, S> {
     ///
     /// ```
     /// use lru::LruCache;
-    /// let mut cache = LruCache::new(2);
+    /// use std::num::NonZeroUsize;
+    /// let mut cache = LruCache::new(NonZeroUsize::new(2).unwrap());
     ///
     /// cache.put(1, "a");
     /// cache.put(2, "b");
@@ -424,7 +435,8 @@ impl<K: Hash + Eq, V, S: BuildHasher> LruCache<K, V, S> {
     ///
     /// ```
     /// use lru::LruCache;
-    /// let mut cache = LruCache::new(2);
+    /// use std::num::NonZeroUsize;
+    /// let mut cache = LruCache::new(NonZeroUsize::new(2).unwrap());
     ///
     /// cache.put("apple", 8);
     /// cache.put("banana", 4);
@@ -454,45 +466,38 @@ impl<K: Hash + Eq, V, S: BuildHasher> LruCache<K, V, S> {
 
     /// Returns a reference to the value of the key in the cache if it is
     /// present in the cache and moves the key to the head of the LRU list.
-    /// If the key does not exist the provided `Fn` is used to populate the list and a reference
-    /// is returned.
-    ///
-    /// This method will only return `None` when the capacity of the cache is 0 and no entries
-    /// can be populated.
+    /// If the key does not exist the provided `FnOnce` is used to populate
+    /// the list and a reference is returned.
     ///
     /// # Example
     ///
     /// ```
     /// use lru::LruCache;
-    /// let mut cache = LruCache::new(2);
+    /// use std::num::NonZeroUsize;
+    /// let mut cache = LruCache::new(NonZeroUsize::new(2).unwrap());
     ///
     /// cache.put(1, "a");
     /// cache.put(2, "b");
     /// cache.put(2, "c");
     /// cache.put(3, "d");
     ///
-    /// assert_eq!(cache.get_or_insert(2, ||"a"), Some(&"c"));
-    /// assert_eq!(cache.get_or_insert(3, ||"a"), Some(&"d"));
-    /// assert_eq!(cache.get_or_insert(1, ||"a"), Some(&"a"));
-    /// assert_eq!(cache.get_or_insert(1, ||"b"), Some(&"a"));
+    /// assert_eq!(cache.get_or_insert(2, ||"a"), &"c");
+    /// assert_eq!(cache.get_or_insert(3, ||"a"), &"d");
+    /// assert_eq!(cache.get_or_insert(1, ||"a"), &"a");
+    /// assert_eq!(cache.get_or_insert(1, ||"b"), &"a");
     /// ```
-    pub fn get_or_insert<'a, F>(&'a mut self, k: K, f: F) -> Option<&'a V>
+    pub fn get_or_insert<'a, F>(&'a mut self, k: K, f: F) -> &'a V
     where
-        F: Fn() -> V,
+        F: FnOnce() -> V,
     {
-        if let Some(node) = self.map.get_mut(&k) {
+        if let Some(node) = self.map.get_mut(&KeyRef { k: &k }) {
             let node_ptr: *mut LruEntry<K, V> = &mut **node;
 
             self.detach(node_ptr);
             self.attach(node_ptr);
 
-            Some(unsafe { &(*(*node_ptr).val.as_ptr()) as &V })
+            unsafe { &(*(*node_ptr).val.as_ptr()) as &V }
         } else {
-            // If the capacity is 0 we do nothing,
-            // this is the only circumstance that should return None
-            if self.cap() == 0 {
-                return None;
-            }
             let v = f();
             let (_, mut node) = self.replace_or_create_node(k, v);
 
@@ -501,7 +506,53 @@ impl<K: Hash + Eq, V, S: BuildHasher> LruCache<K, V, S> {
 
             let keyref = unsafe { (*node_ptr).key.as_ptr() };
             self.map.insert(KeyRef { k: keyref }, node);
-            Some(unsafe { &(*(*node_ptr).val.as_ptr()) as &V })
+            unsafe { &(*(*node_ptr).val.as_ptr()) as &V }
+        }
+    }
+
+    /// Returns a mutable reference to the value of the key in the cache if it is
+    /// present in the cache and moves the key to the head of the LRU list.
+    /// If the key does not exist the provided `FnOnce` is used to populate
+    /// the list and a mutable reference is returned.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// use lru::LruCache;
+    /// use std::num::NonZeroUsize;
+    /// let mut cache = LruCache::new(NonZeroUsize::new(2).unwrap());
+    ///
+    /// cache.put(1, "a");
+    /// cache.put(2, "b");
+    ///
+    /// let v = cache.get_or_insert_mut(2, ||"c");
+    /// assert_eq!(v, &"b");
+    /// *v = "d";
+    /// assert_eq!(cache.get_or_insert_mut(2, ||"e"), &mut "d");
+    /// assert_eq!(cache.get_or_insert_mut(3, ||"f"), &mut "f");
+    /// assert_eq!(cache.get_or_insert_mut(3, ||"e"), &mut "f");
+    /// ```
+    pub fn get_or_insert_mut<'a, F>(&'a mut self, k: K, f: F) -> &'a mut V
+    where
+        F: FnOnce() -> V,
+    {
+        if let Some(node) = self.map.get_mut(&KeyRef { k: &k }) {
+            let node_ptr: *mut LruEntry<K, V> = &mut **node;
+
+            self.detach(node_ptr);
+            self.attach(node_ptr);
+
+            unsafe { &mut (*(*node_ptr).val.as_mut_ptr()) as &mut V }
+        } else {
+            let v = f();
+            let (_, mut node) = self.replace_or_create_node(k, v);
+
+            let node_ptr: *mut LruEntry<K, V> = &mut *node;
+            self.attach(node_ptr);
+
+            let keyref = unsafe { (*node_ptr).key.as_ptr() };
+            self.map.insert(KeyRef { k: keyref }, node);
+            unsafe { &mut (*(*node_ptr).val.as_mut_ptr()) as &mut V }
         }
     }
 
@@ -513,7 +564,8 @@ impl<K: Hash + Eq, V, S: BuildHasher> LruCache<K, V, S> {
     ///
     /// ```
     /// use lru::LruCache;
-    /// let mut cache = LruCache::new(2);
+    /// use std::num::NonZeroUsize;
+    /// let mut cache = LruCache::new(NonZeroUsize::new(2).unwrap());
     ///
     /// cache.put(1, "a");
     /// cache.put(2, "b");
@@ -528,7 +580,7 @@ impl<K: Hash + Eq, V, S: BuildHasher> LruCache<K, V, S> {
     {
         self.map
             .get(k)
-            .map(|node| unsafe { &(*(*node).val.as_ptr()) as &V })
+            .map(|node| unsafe { &(*node.val.as_ptr()) as &V })
     }
 
     /// Returns a mutable reference to the value corresponding to the key in the cache or `None`
@@ -539,7 +591,8 @@ impl<K: Hash + Eq, V, S: BuildHasher> LruCache<K, V, S> {
     ///
     /// ```
     /// use lru::LruCache;
-    /// let mut cache = LruCache::new(2);
+    /// use std::num::NonZeroUsize;
+    /// let mut cache = LruCache::new(NonZeroUsize::new(2).unwrap());
     ///
     /// cache.put(1, "a");
     /// cache.put(2, "b");
@@ -554,7 +607,7 @@ impl<K: Hash + Eq, V, S: BuildHasher> LruCache<K, V, S> {
     {
         match self.map.get_mut(k) {
             None => None,
-            Some(node) => Some(unsafe { &mut (*(*node).val.as_mut_ptr()) as &mut V }),
+            Some(node) => Some(unsafe { &mut (*node.val.as_mut_ptr()) as &mut V }),
         }
     }
 
@@ -566,14 +619,15 @@ impl<K: Hash + Eq, V, S: BuildHasher> LruCache<K, V, S> {
     ///
     /// ```
     /// use lru::LruCache;
-    /// let mut cache = LruCache::new(2);
+    /// use std::num::NonZeroUsize;
+    /// let mut cache = LruCache::new(NonZeroUsize::new(2).unwrap());
     ///
     /// cache.put(1, "a");
     /// cache.put(2, "b");
     ///
     /// assert_eq!(cache.peek_lru(), Some((&1, &"a")));
     /// ```
-    pub fn peek_lru<'a>(&'_ self) -> Option<(&'a K, &'a V)> {
+    pub fn peek_lru<'a>(&'a self) -> Option<(&'a K, &'a V)> {
         if self.is_empty() {
             return None;
         }
@@ -595,7 +649,8 @@ impl<K: Hash + Eq, V, S: BuildHasher> LruCache<K, V, S> {
     ///
     /// ```
     /// use lru::LruCache;
-    /// let mut cache = LruCache::new(2);
+    /// use std::num::NonZeroUsize;
+    /// let mut cache = LruCache::new(NonZeroUsize::new(2).unwrap());
     ///
     /// cache.put(1, "a");
     /// cache.put(2, "b");
@@ -620,7 +675,8 @@ impl<K: Hash + Eq, V, S: BuildHasher> LruCache<K, V, S> {
     ///
     /// ```
     /// use lru::LruCache;
-    /// let mut cache = LruCache::new(2);
+    /// use std::num::NonZeroUsize;
+    /// let mut cache = LruCache::new(NonZeroUsize::new(2).unwrap());
     ///
     /// cache.put(2, "a");
     ///
@@ -647,6 +703,40 @@ impl<K: Hash + Eq, V, S: BuildHasher> LruCache<K, V, S> {
         }
     }
 
+    /// Removes and returns the key and the value corresponding to the key from the cache or
+    /// `None` if it does not exist.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// use lru::LruCache;
+    /// use std::num::NonZeroUsize;
+    /// let mut cache = LruCache::new(NonZeroUsize::new(2).unwrap());
+    ///
+    /// cache.put(1, "a");
+    /// cache.put(2, "a");
+    ///
+    /// assert_eq!(cache.pop(&1), Some("a"));
+    /// assert_eq!(cache.pop_entry(&2), Some((2, "a")));
+    /// assert_eq!(cache.pop(&1), None);
+    /// assert_eq!(cache.pop_entry(&2), None);
+    /// assert_eq!(cache.len(), 0);
+    /// ```
+    pub fn pop_entry<Q>(&mut self, k: &Q) -> Option<(K, V)>
+    where
+        KeyRef<K>: Borrow<Q>,
+        Q: Hash + Eq + ?Sized,
+    {
+        match self.map.remove(k) {
+            None => None,
+            Some(mut old_node) => {
+                let node_ptr: *mut LruEntry<K, V> = &mut *old_node;
+                self.detach(node_ptr);
+                unsafe { Some((old_node.key.assume_init(), old_node.val.assume_init())) }
+            }
+        }
+    }
+
     /// Removes and returns the key and value corresponding to the least recently
     /// used item or `None` if the cache is empty.
     ///
@@ -654,7 +744,8 @@ impl<K: Hash + Eq, V, S: BuildHasher> LruCache<K, V, S> {
     ///
     /// ```
     /// use lru::LruCache;
-    /// let mut cache = LruCache::new(2);
+    /// use std::num::NonZeroUsize;
+    /// let mut cache = LruCache::new(NonZeroUsize::new(2).unwrap());
     ///
     /// cache.put(2, "a");
     /// cache.put(3, "b");
@@ -674,13 +765,84 @@ impl<K: Hash + Eq, V, S: BuildHasher> LruCache<K, V, S> {
         unsafe { Some((key.assume_init(), val.assume_init())) }
     }
 
+    /// Marks the key as the most recently used one.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// use lru::LruCache;
+    /// use std::num::NonZeroUsize;
+    /// let mut cache = LruCache::new(NonZeroUsize::new(3).unwrap());
+    ///
+    /// cache.put(1, "a");
+    /// cache.put(2, "b");
+    /// cache.put(3, "c");
+    /// cache.get(&1);
+    /// cache.get(&2);
+    ///
+    /// // If we do `pop_lru` now, we would pop 3.
+    /// // assert_eq!(cache.pop_lru(), Some((3, "c")));
+    ///
+    /// // By promoting 3, we make sure it isn't popped.
+    /// cache.promote(&3);
+    /// assert_eq!(cache.pop_lru(), Some((1, "a")));
+    /// ```
+    pub fn promote<'a, Q>(&'a mut self, k: &Q)
+    where
+        KeyRef<K>: Borrow<Q>,
+        Q: Hash + Eq + ?Sized,
+    {
+        if let Some(node) = self.map.get_mut(k) {
+            let node_ptr: *mut LruEntry<K, V> = &mut **node;
+            self.detach(node_ptr);
+            self.attach(node_ptr);
+        }
+    }
+
+    /// Marks the key as the least recently used one.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// use lru::LruCache;
+    /// use std::num::NonZeroUsize;
+    /// let mut cache = LruCache::new(NonZeroUsize::new(3).unwrap());
+    ///
+    /// cache.put(1, "a");
+    /// cache.put(2, "b");
+    /// cache.put(3, "c");
+    /// cache.get(&1);
+    /// cache.get(&2);
+    ///
+    /// // If we do `pop_lru` now, we would pop 3.
+    /// // assert_eq!(cache.pop_lru(), Some((3, "c")));
+    ///
+    /// // By demoting 1 and 2, we make sure those are popped first.
+    /// cache.demote(&2);
+    /// cache.demote(&1);
+    /// assert_eq!(cache.pop_lru(), Some((1, "a")));
+    /// assert_eq!(cache.pop_lru(), Some((2, "b")));
+    /// ```
+    pub fn demote<'a, Q>(&'a mut self, k: &Q)
+    where
+        KeyRef<K>: Borrow<Q>,
+        Q: Hash + Eq + ?Sized,
+    {
+        if let Some(node) = self.map.get_mut(k) {
+            let node_ptr: *mut LruEntry<K, V> = &mut **node;
+            self.detach(node_ptr);
+            self.attach_last(node_ptr);
+        }
+    }
+
     /// Returns the number of key-value pairs that are currently in the the cache.
     ///
     /// # Example
     ///
     /// ```
     /// use lru::LruCache;
-    /// let mut cache = LruCache::new(2);
+    /// use std::num::NonZeroUsize;
+    /// let mut cache = LruCache::new(NonZeroUsize::new(2).unwrap());
     /// assert_eq!(cache.len(), 0);
     ///
     /// cache.put(1, "a");
@@ -702,7 +864,8 @@ impl<K: Hash + Eq, V, S: BuildHasher> LruCache<K, V, S> {
     ///
     /// ```
     /// use lru::LruCache;
-    /// let mut cache = LruCache::new(2);
+    /// use std::num::NonZeroUsize;
+    /// let mut cache = LruCache::new(NonZeroUsize::new(2).unwrap());
     /// assert!(cache.is_empty());
     ///
     /// cache.put(1, "a");
@@ -718,10 +881,11 @@ impl<K: Hash + Eq, V, S: BuildHasher> LruCache<K, V, S> {
     ///
     /// ```
     /// use lru::LruCache;
-    /// let mut cache: LruCache<isize, &str> = LruCache::new(2);
-    /// assert_eq!(cache.cap(), 2);
+    /// use std::num::NonZeroUsize;
+    /// let mut cache: LruCache<isize, &str> = LruCache::new(NonZeroUsize::new(2).unwrap());
+    /// assert_eq!(cache.cap().get(), 2);
     /// ```
-    pub fn cap(&self) -> usize {
+    pub fn cap(&self) -> NonZeroUsize {
         self.cap
     }
 
@@ -732,11 +896,12 @@ impl<K: Hash + Eq, V, S: BuildHasher> LruCache<K, V, S> {
     ///
     /// ```
     /// use lru::LruCache;
-    /// let mut cache: LruCache<isize, &str> = LruCache::new(2);
+    /// use std::num::NonZeroUsize;
+    /// let mut cache: LruCache<isize, &str> = LruCache::new(NonZeroUsize::new(2).unwrap());
     ///
     /// cache.put(1, "a");
     /// cache.put(2, "b");
-    /// cache.resize(4);
+    /// cache.resize(NonZeroUsize::new(4).unwrap());
     /// cache.put(3, "c");
     /// cache.put(4, "d");
     ///
@@ -746,13 +911,13 @@ impl<K: Hash + Eq, V, S: BuildHasher> LruCache<K, V, S> {
     /// assert_eq!(cache.get(&3), Some(&"c"));
     /// assert_eq!(cache.get(&4), Some(&"d"));
     /// ```
-    pub fn resize(&mut self, cap: usize) {
+    pub fn resize(&mut self, cap: NonZeroUsize) {
         // return early if capacity doesn't change
         if cap == self.cap {
             return;
         }
 
-        while self.map.len() > cap {
+        while self.map.len() > cap.get() {
             self.pop_lru();
         }
         self.map.shrink_to_fit();
@@ -766,7 +931,8 @@ impl<K: Hash + Eq, V, S: BuildHasher> LruCache<K, V, S> {
     ///
     /// ```
     /// use lru::LruCache;
-    /// let mut cache: LruCache<isize, &str> = LruCache::new(2);
+    /// use std::num::NonZeroUsize;
+    /// let mut cache: LruCache<isize, &str> = LruCache::new(NonZeroUsize::new(2).unwrap());
     /// assert_eq!(cache.len(), 0);
     ///
     /// cache.put(1, "a");
@@ -789,8 +955,9 @@ impl<K: Hash + Eq, V, S: BuildHasher> LruCache<K, V, S> {
     ///
     /// ```
     /// use lru::LruCache;
+    /// use std::num::NonZeroUsize;
     ///
-    /// let mut cache = LruCache::new(3);
+    /// let mut cache = LruCache::new(NonZeroUsize::new(3).unwrap());
     /// cache.put("a", 1);
     /// cache.put("b", 2);
     /// cache.put("c", 3);
@@ -815,13 +982,14 @@ impl<K: Hash + Eq, V, S: BuildHasher> LruCache<K, V, S> {
     ///
     /// ```
     /// use lru::LruCache;
+    /// use std::num::NonZeroUsize;
     ///
     /// struct HddBlock {
     ///     dirty: bool,
     ///     data: [u8; 512]
     /// }
     ///
-    /// let mut cache = LruCache::new(3);
+    /// let mut cache = LruCache::new(NonZeroUsize::new(3).unwrap());
     /// cache.put(0, HddBlock { dirty: false, data: [0x00; 512]});
     /// cache.put(1, HddBlock { dirty: true,  data: [0x55; 512]});
     /// cache.put(2, HddBlock { dirty: true,  data: [0x77; 512]});
@@ -866,6 +1034,7 @@ impl<K: Hash + Eq, V, S: BuildHasher> LruCache<K, V, S> {
         }
     }
 
+    // Attaches `node` after the sigil `self.head` node.
     fn attach(&mut self, node: *mut LruEntry<K, V>) {
         unsafe {
             (*node).next = (*self.head).next;
@@ -874,6 +1043,16 @@ impl<K: Hash + Eq, V, S: BuildHasher> LruCache<K, V, S> {
             (*(*node).next).prev = node;
         }
     }
+
+    // Attaches `node` before the sigil `self.tail` node.
+    fn attach_last(&mut self, node: *mut LruEntry<K, V>) {
+        unsafe {
+            (*node).next = self.tail;
+            (*node).prev = (*self.tail).prev;
+            (*self.tail).prev = node;
+            (*(*node).prev).next = node;
+        }
+    }
 }
 
 impl<K, V, S> Drop for LruCache<K, V, S> {
@@ -884,10 +1063,9 @@ impl<K, V, S> Drop for LruCache<K, V, S> {
         });
         // We rebox the head/tail, and because these are maybe-uninit
         // they do not have the absent k/v dropped.
-        unsafe {
-            let _head = *Box::from_raw(self.head);
-            let _tail = *Box::from_raw(self.tail);
-        }
+
+        let _head = unsafe { *Box::from_raw(self.head) };
+        let _tail = unsafe { *Box::from_raw(self.tail) };
     }
 }
 
@@ -1102,21 +1280,21 @@ where
 }
 
 impl<K, V> ExactSizeIterator for IntoIter<K, V> where K: Hash + Eq {}
-impl<K, V> FusedIterator for IntoIter<K, V> where K: Hash + Eq  {}
+impl<K, V> FusedIterator for IntoIter<K, V> where K: Hash + Eq {}
 
 impl<K: Hash + Eq, V> IntoIterator for LruCache<K, V> {
     type Item = (K, V);
     type IntoIter = IntoIter<K, V>;
 
     fn into_iter(self) -> IntoIter<K, V> {
-        IntoIter{cache: self}
+        IntoIter { cache: self }
     }
 }
 
 #[cfg(test)]
 mod tests {
     use super::LruCache;
-    use core::fmt::Debug;
+    use core::{fmt::Debug, num::NonZeroUsize};
     use scoped_threadpool::Pool;
     use std::sync::atomic::{AtomicUsize, Ordering};
 
@@ -1162,10 +1340,12 @@ mod tests {
     #[test]
     #[cfg(feature = "hashbrown")]
     fn test_with_hasher() {
+        use core::num::NonZeroUsize;
+
         use hashbrown::hash_map::DefaultHashBuilder;
 
         let s = DefaultHashBuilder::default();
-        let mut cache = LruCache::with_hasher(16, s);
+        let mut cache = LruCache::with_hasher(NonZeroUsize::new(16).unwrap(), s);
 
         for i in 0..13370 {
             cache.put(i, ());
@@ -1175,13 +1355,13 @@ mod tests {
 
     #[test]
     fn test_put_and_get() {
-        let mut cache = LruCache::new(2);
+        let mut cache = LruCache::new(NonZeroUsize::new(2).unwrap());
         assert!(cache.is_empty());
 
         assert_eq!(cache.put("apple", "red"), None);
         assert_eq!(cache.put("banana", "yellow"), None);
 
-        assert_eq!(cache.cap(), 2);
+        assert_eq!(cache.cap().get(), 2);
         assert_eq!(cache.len(), 2);
         assert!(!cache.is_empty());
         assert_opt_eq(cache.get(&"apple"), "red");
@@ -1190,29 +1370,50 @@ mod tests {
 
     #[test]
     fn test_put_and_get_or_insert() {
-        let mut cache = LruCache::new(2);
+        let mut cache = LruCache::new(NonZeroUsize::new(2).unwrap());
         assert!(cache.is_empty());
 
         assert_eq!(cache.put("apple", "red"), None);
         assert_eq!(cache.put("banana", "yellow"), None);
 
-        assert_eq!(cache.cap(), 2);
+        assert_eq!(cache.cap().get(), 2);
         assert_eq!(cache.len(), 2);
         assert!(!cache.is_empty());
-        assert_opt_eq(cache.get_or_insert(&"apple", || "orange"), &"red");
-        assert_opt_eq(cache.get_or_insert(&"banana", || "orange"), &"yellow");
-        assert_opt_eq(cache.get_or_insert(&"lemon", || "orange"), &"orange");
-        assert_opt_eq(cache.get_or_insert(&"lemon", || "red"), &"orange");
+        assert_eq!(cache.get_or_insert("apple", || "orange"), &"red");
+        assert_eq!(cache.get_or_insert("banana", || "orange"), &"yellow");
+        assert_eq!(cache.get_or_insert("lemon", || "orange"), &"orange");
+        assert_eq!(cache.get_or_insert("lemon", || "red"), &"orange");
+    }
+
+    #[test]
+    fn test_put_and_get_or_insert_mut() {
+        let mut cache = LruCache::new(NonZeroUsize::new(2).unwrap());
+        assert!(cache.is_empty());
+
+        assert_eq!(cache.put("apple", "red"), None);
+        assert_eq!(cache.put("banana", "yellow"), None);
+
+        assert_eq!(cache.cap().get(), 2);
+        assert_eq!(cache.len(), 2);
+
+        let v = cache.get_or_insert_mut("apple", || "orange");
+        assert_eq!(v, &"red");
+        *v = "blue";
+
+        assert_eq!(cache.get_or_insert_mut("apple", || "orange"), &"blue");
+        assert_eq!(cache.get_or_insert_mut("banana", || "orange"), &"yellow");
+        assert_eq!(cache.get_or_insert_mut("lemon", || "orange"), &"orange");
+        assert_eq!(cache.get_or_insert_mut("lemon", || "red"), &"orange");
     }
 
     #[test]
     fn test_put_and_get_mut() {
-        let mut cache = LruCache::new(2);
+        let mut cache = LruCache::new(NonZeroUsize::new(2).unwrap());
 
         cache.put("apple", "red");
         cache.put("banana", "yellow");
 
-        assert_eq!(cache.cap(), 2);
+        assert_eq!(cache.cap().get(), 2);
         assert_eq!(cache.len(), 2);
         assert_opt_eq_mut(cache.get_mut(&"apple"), "red");
         assert_opt_eq_mut(cache.get_mut(&"banana"), "yellow");
@@ -1220,7 +1421,7 @@ mod tests {
 
     #[test]
     fn test_get_mut_and_update() {
-        let mut cache = LruCache::new(2);
+        let mut cache = LruCache::new(NonZeroUsize::new(2).unwrap());
 
         cache.put("apple", 1);
         cache.put("banana", 3);
@@ -1230,7 +1431,7 @@ mod tests {
             *v = 4;
         }
 
-        assert_eq!(cache.cap(), 2);
+        assert_eq!(cache.cap().get(), 2);
         assert_eq!(cache.len(), 2);
         assert_opt_eq_mut(cache.get_mut(&"apple"), 4);
         assert_opt_eq_mut(cache.get_mut(&"banana"), 3);
@@ -1238,7 +1439,7 @@ mod tests {
 
     #[test]
     fn test_put_update() {
-        let mut cache = LruCache::new(1);
+        let mut cache = LruCache::new(NonZeroUsize::new(2).unwrap());
 
         assert_eq!(cache.put("apple", "red"), None);
         assert_eq!(cache.put("apple", "green"), Some("red"));
@@ -1249,7 +1450,7 @@ mod tests {
 
     #[test]
     fn test_put_removes_oldest() {
-        let mut cache = LruCache::new(2);
+        let mut cache = LruCache::new(NonZeroUsize::new(2).unwrap());
 
         assert_eq!(cache.put("apple", "red"), None);
         assert_eq!(cache.put("banana", "yellow"), None);
@@ -1271,7 +1472,7 @@ mod tests {
 
     #[test]
     fn test_peek() {
-        let mut cache = LruCache::new(2);
+        let mut cache = LruCache::new(NonZeroUsize::new(2).unwrap());
 
         cache.put("apple", "red");
         cache.put("banana", "yellow");
@@ -1288,7 +1489,7 @@ mod tests {
 
     #[test]
     fn test_peek_mut() {
-        let mut cache = LruCache::new(2);
+        let mut cache = LruCache::new(NonZeroUsize::new(2).unwrap());
 
         cache.put("apple", "red");
         cache.put("banana", "yellow");
@@ -1313,7 +1514,7 @@ mod tests {
 
     #[test]
     fn test_peek_lru() {
-        let mut cache = LruCache::new(2);
+        let mut cache = LruCache::new(NonZeroUsize::new(2).unwrap());
 
         assert!(cache.peek_lru().is_none());
 
@@ -1330,7 +1531,7 @@ mod tests {
 
     #[test]
     fn test_contains() {
-        let mut cache = LruCache::new(2);
+        let mut cache = LruCache::new(NonZeroUsize::new(2).unwrap());
 
         cache.put("apple", "red");
         cache.put("banana", "yellow");
@@ -1343,7 +1544,7 @@ mod tests {
 
     #[test]
     fn test_pop() {
-        let mut cache = LruCache::new(2);
+        let mut cache = LruCache::new(NonZeroUsize::new(2).unwrap());
 
         cache.put("apple", "red");
         cache.put("banana", "yellow");
@@ -1360,9 +1561,27 @@ mod tests {
         assert_opt_eq(cache.get(&"banana"), "yellow");
     }
 
+    #[test]
+    fn test_pop_entry() {
+        let mut cache = LruCache::new(NonZeroUsize::new(2).unwrap());
+        cache.put("apple", "red");
+        cache.put("banana", "yellow");
+
+        assert_eq!(cache.len(), 2);
+        assert_opt_eq(cache.get(&"apple"), "red");
+        assert_opt_eq(cache.get(&"banana"), "yellow");
+
+        let popped = cache.pop_entry(&"apple");
+        assert!(popped.is_some());
+        assert_eq!(popped.unwrap(), ("apple", "red"));
+        assert_eq!(cache.len(), 1);
+        assert!(cache.get(&"apple").is_none());
+        assert_opt_eq(cache.get(&"banana"), "yellow");
+    }
+
     #[test]
     fn test_pop_lru() {
-        let mut cache = LruCache::new(200);
+        let mut cache = LruCache::new(NonZeroUsize::new(200).unwrap());
 
         for i in 0..75 {
             cache.put(i, "A");
@@ -1397,7 +1616,7 @@ mod tests {
 
     #[test]
     fn test_clear() {
-        let mut cache = LruCache::new(2);
+        let mut cache = LruCache::new(NonZeroUsize::new(2).unwrap());
 
         cache.put("apple", "red");
         cache.put("banana", "yellow");
@@ -1412,11 +1631,11 @@ mod tests {
 
     #[test]
     fn test_resize_larger() {
-        let mut cache = LruCache::new(2);
+        let mut cache = LruCache::new(NonZeroUsize::new(2).unwrap());
 
         cache.put(1, "a");
         cache.put(2, "b");
-        cache.resize(4);
+        cache.resize(NonZeroUsize::new(4).unwrap());
         cache.put(3, "c");
         cache.put(4, "d");
 
@@ -1429,14 +1648,14 @@ mod tests {
 
     #[test]
     fn test_resize_smaller() {
-        let mut cache = LruCache::new(4);
+        let mut cache = LruCache::new(NonZeroUsize::new(4).unwrap());
 
         cache.put(1, "a");
         cache.put(2, "b");
         cache.put(3, "c");
         cache.put(4, "d");
 
-        cache.resize(2);
+        cache.resize(NonZeroUsize::new(2).unwrap());
 
         assert_eq!(cache.len(), 2);
         assert!(cache.get(&1).is_none());
@@ -1449,7 +1668,7 @@ mod tests {
     fn test_send() {
         use std::thread;
 
-        let mut cache = LruCache::new(4);
+        let mut cache = LruCache::new(NonZeroUsize::new(4).unwrap());
         cache.put(1, "a");
 
         let handle = thread::spawn(move || {
@@ -1462,7 +1681,7 @@ mod tests {
     #[test]
     fn test_multiple_threads() {
         let mut pool = Pool::new(1);
-        let mut cache = LruCache::new(4);
+        let mut cache = LruCache::new(NonZeroUsize::new(4).unwrap());
         cache.put(1, "a");
 
         let cache_ref = &cache;
@@ -1477,7 +1696,7 @@ mod tests {
 
     #[test]
     fn test_iter_forwards() {
-        let mut cache = LruCache::new(3);
+        let mut cache = LruCache::new(NonZeroUsize::new(3).unwrap());
         cache.put("a", 1);
         cache.put("b", 2);
         cache.put("c", 3);
@@ -1516,7 +1735,7 @@ mod tests {
 
     #[test]
     fn test_iter_backwards() {
-        let mut cache = LruCache::new(3);
+        let mut cache = LruCache::new(NonZeroUsize::new(3).unwrap());
         cache.put("a", 1);
         cache.put("b", 2);
         cache.put("c", 3);
@@ -1556,7 +1775,7 @@ mod tests {
 
     #[test]
     fn test_iter_forwards_and_backwards() {
-        let mut cache = LruCache::new(3);
+        let mut cache = LruCache::new(NonZeroUsize::new(3).unwrap());
         cache.put("a", 1);
         cache.put("b", 2);
         cache.put("c", 3);
@@ -1596,7 +1815,7 @@ mod tests {
     #[test]
     fn test_iter_multiple_threads() {
         let mut pool = Pool::new(1);
-        let mut cache = LruCache::new(3);
+        let mut cache = LruCache::new(NonZeroUsize::new(3).unwrap());
         cache.put("a", 1);
         cache.put("b", 2);
         cache.put("c", 3);
@@ -1624,7 +1843,7 @@ mod tests {
 
     #[test]
     fn test_iter_clone() {
-        let mut cache = LruCache::new(3);
+        let mut cache = LruCache::new(NonZeroUsize::new(3).unwrap());
         cache.put("a", 1);
         cache.put("b", 2);
 
@@ -1649,7 +1868,7 @@ mod tests {
 
     #[test]
     fn test_into_iter() {
-        let mut cache = LruCache::new(3);
+        let mut cache = LruCache::new(NonZeroUsize::new(3).unwrap());
         cache.put("a", 1);
         cache.put("b", 2);
         cache.put("c", 3);
@@ -1670,7 +1889,7 @@ mod tests {
 
     #[test]
     fn test_that_pop_actually_detaches_node() {
-        let mut cache = LruCache::new(5);
+        let mut cache = LruCache::new(NonZeroUsize::new(5).unwrap());
 
         cache.put("a", 1);
         cache.put("b", 2);
@@ -1695,7 +1914,7 @@ mod tests {
     fn test_get_with_borrow() {
         use alloc::string::String;
 
-        let mut cache = LruCache::new(2);
+        let mut cache = LruCache::new(NonZeroUsize::new(2).unwrap());
 
         let key = String::from("apple");
         cache.put(key, "red");
@@ -1707,7 +1926,7 @@ mod tests {
     fn test_get_mut_with_borrow() {
         use alloc::string::String;
 
-        let mut cache = LruCache::new(2);
+        let mut cache = LruCache::new(NonZeroUsize::new(2).unwrap());
 
         let key = String::from("apple");
         cache.put(key, "red");
@@ -1729,7 +1948,7 @@ mod tests {
 
         let n = 100;
         for _ in 0..n {
-            let mut cache = LruCache::new(1);
+            let mut cache = LruCache::new(NonZeroUsize::new(1).unwrap());
             for i in 0..n {
                 cache.put(i, DropCounter {});
             }
@@ -1751,7 +1970,7 @@ mod tests {
 
         let n = 100;
         for _ in 0..n {
-            let mut cache = LruCache::new(1);
+            let mut cache = LruCache::new(NonZeroUsize::new(1).unwrap());
             for i in 0..n {
                 cache.put(i, DropCounter {});
             }
@@ -1774,11 +1993,11 @@ mod tests {
 
         let n = 100;
         for _ in 0..n {
-            let mut cache = LruCache::new(1);
+            let mut cache = LruCache::new(NonZeroUsize::new(1).unwrap());
             for i in 0..n {
                 cache.put(i, DropCounter {});
             }
-            cache.resize(0);
+            cache.clear();
         }
         assert_eq!(DROP_COUNT.load(Ordering::SeqCst), n * n);
     }
@@ -1804,7 +2023,7 @@ mod tests {
 
         let n = 100;
         for _ in 0..n {
-            let mut cache = LruCache::new(1);
+            let mut cache = LruCache::new(NonZeroUsize::new(1).unwrap());
 
             for i in 0..100 {
                 cache.put(KeyDropCounter(i), i);
@@ -1816,8 +2035,34 @@ mod tests {
     }
 
     #[test]
-    fn test_zero_cap_no_crash() {
-        let mut cache = LruCache::new(0);
-        cache.put("reizeiin", "tohka");
+    fn test_promote_and_demote() {
+        let mut cache = LruCache::new(NonZeroUsize::new(5).unwrap());
+        for i in 0..5 {
+            cache.push(i, i);
+        }
+        cache.promote(&1);
+        cache.promote(&0);
+        cache.demote(&3);
+        cache.demote(&4);
+        assert_eq!(cache.pop_lru(), Some((4, 4)));
+        assert_eq!(cache.pop_lru(), Some((3, 3)));
+        assert_eq!(cache.pop_lru(), Some((2, 2)));
+        assert_eq!(cache.pop_lru(), Some((1, 1)));
+        assert_eq!(cache.pop_lru(), Some((0, 0)));
+        assert_eq!(cache.pop_lru(), None);
     }
 }
+
+/// Doctests for what should *not* compile
+///
+/// ```compile_fail
+/// let mut cache = lru::LruCache::<u32, u32>::unbounded();
+/// let _: &'static u32 = cache.get_or_insert(0, || 92);
+/// ```
+///
+/// ```compile_fail
+/// let mut cache = lru::LruCache::<u32, u32>::unbounded();
+/// let _: Option<(&'static u32, _)> = cache.peek_lru();
+/// let _: Option<(_, &'static u32)> = cache.peek_lru();
+/// ```
+fn _test_lifetimes() {}
